import * as mc from "@minecraft/server";
import * as mcui from "@minecraft/server-ui";


///Variables
var tubeId = "dnd:item_transporter"
var tubesLimit = 24
var allowedConnections = [
    "minecraft:chest",
    "minecraft:barrel",
    "bedrock:accept_item_transporter"
]
//Functions
function Vector3(x, y, z) {
    let Vector = {
        x: x,
        y: y,
        z: z
    }
    return Vector
}
function getFace(block, face) {
    let state = "default"
    if (face != "closed") {
        if (block != undefined) {
            state = "open"
        } else if (block == undefined) {
            state = "default"
        }
    } else if (face == "closed") {
        state = "closed"
    }
    return state
}
function getBlock(x, y, z, block = mc.world.getDimension("overworld").getBlock(0, 0, 0)) {
    let blockT = block.dimension.getBlock(Vector3(block.location.x + x, block.location.y + y, block.location.z + z))
    return blockT
}
function blockInConnections(block) {
    if (block) {
        if (allowedConnections.includes(block.typeId)) {
            return block
        } else if (block.getTags().filter(tag => allowedConnections.includes(tag)).length > 0) {
            return block
        } else if (!allowedConnections.includes(block.typeId) && block.getTags().filter(tag => allowedConnections.includes(tag)).length == 0) {
            return undefined
        }
    }
    return undefined
}
async function PipeDetectAndTransfer(blockTarget, lastid, done, data, start, container) {
    let blocks = data
    const id = `${blockTarget.x} ${blockTarget.y} ${blockTarget.z}`;
    if (done.has(id)) {
        return;
    }
    done.add(id);

    if (start && blocks.length < tubesLimit && blockTarget.typeId == lastid) {
        let offsets = [
            getBlock(0, 0, -1, blockTarget),
            getBlock(0, 0, 1, blockTarget),
            getBlock(1, 0, 0, blockTarget),
            getBlock(-1, 0, 0, blockTarget),
            getBlock(0, 1, 0, blockTarget),
            getBlock(0, -1, 0, blockTarget)
        ]

        blocks.push(blockTarget)
        if (offsets[0].permutation.getState("dnd:south") == "open") {
            PipeDetectAndTransfer(offsets[0], lastid, done, blocks, start, container);
        }
        if (offsets[1].permutation.getState("dnd:north") == "open") {
            PipeDetectAndTransfer(offsets[1], lastid, done, blocks, start, container);
        }
        if (offsets[2].permutation.getState("dnd:west") == "open") {
            PipeDetectAndTransfer(offsets[2], lastid, done, blocks, start, container);
        }
        if (offsets[3].permutation.getState("dnd:east") == "open") {
            PipeDetectAndTransfer(offsets[3], lastid, done, blocks, start, container);
        }
        if (offsets[4].permutation.getState("dnd:bottom") == "open") {
            PipeDetectAndTransfer(offsets[4], lastid, done, blocks, start, container);
        }
        if (offsets[5].permutation.getState("dnd:top") == "open") {
            PipeDetectAndTransfer(offsets[5], lastid, done, blocks, start, container);
        }

    }
    TransportItem(blockTarget, container)
}
async function TransportItem(pipe = mc.Block, container) {
    let faces = [
        "dnd:north",
        "dnd:south",
        "dnd:east",
        "dnd:west",
        "dnd:top",
        "dnd:bottom"
    ]
    let actualStates = pipe.permutation.clone().getAllStates()
    let blocksOffsets = [
        blockInConnections(getBlock(0, 0, -1, pipe)),
        blockInConnections(getBlock(0, 0, 1, pipe)),
        blockInConnections(getBlock(1, 0, 0, pipe)),
        blockInConnections(getBlock(-1, 0, 0, pipe)),
        blockInConnections(getBlock(0, 1, 0, pipe)),
        blockInConnections(getBlock(0, -1, 0, pipe))
    ]
    faces.forEach((face, index) => {
        if (actualStates[face] == "open" && blocksOffsets[index]) {
            if (blocksOffsets[index].getComponent("inventory")) {
                let targetContainer = blocksOffsets[index].getComponent("inventory").container
                for (let s = 0; s < container.size; s++) {
                    container.transferItem(s, targetContainer)
                }
            } else {
                let entityContainer = pipe.dimension.getEntitiesAtBlockLocation(blocksOffsets[index].location)[0]
                if (entityContainer) {
                    if (entityContainer.getComponent("inventory")) {
                        let targetContainer = entityContainer.getComponent("inventory").container
                        for (let s = 0; s < container.size; s++) {
                            container.transferItem(s, targetContainer)
                        }
                    }

                }
            }
        }
    })
}
function ItemTransport(pipe, blocksOffsets, actualStates) {
    let faces = [
        "dnd:north",
        "dnd:south",
        "dnd:east",
        "dnd:west",
        "dnd:top",
        "dnd:bottom"
    ]
    faces.forEach((face, index) => {
        if ((actualStates[face] == "output" && blocksOffsets[index])) {
            if (blocksOffsets[index].getComponent("inventory")) {
                PipeDetectAndTransfer(pipe, pipe.typeId, new Set(), [], true, blocksOffsets[index].getComponent("inventory").container);
            } else {
                let entityContainer = pipe.dimension.getEntitiesAtBlockLocation(blocksOffsets[index].location)[0]
                if (entityContainer) {
                    if (entityContainer.getComponent("inventory")) {
                        PipeDetectAndTransfer(pipe, pipe.typeId, new Set(), [], true, entityContainer.getComponent("inventory").container);
                    }

                }
            }
        }
    })
}
///Events
mc.system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id == "dnd:item_transporter") {
        let pipe = data.sourceBlock
        if (pipe && pipe.typeId == tubeId) {
            let actualStates = pipe.permutation.getAllStates()
            // let block = mc.BlockPermutation.resolve(pipe.typeId, actualStates)
            let blocksOffsets = [
                blockInConnections(getBlock(0, 0, -1, pipe)),
                blockInConnections(getBlock(0, 0, 1, pipe)),
                blockInConnections(getBlock(1, 0, 0, pipe)),
                blockInConnections(getBlock(-1, 0, 0, pipe)),
                blockInConnections(getBlock(0, 1, 0, pipe)),
                blockInConnections(getBlock(0, -1, 0, pipe))
            ]
            let faces = [
                "dnd:north",
                "dnd:south",
                "dnd:east",
                "dnd:west",
                "dnd:top",
                "dnd:bottom"
            ]
            faces.forEach((face, index) => {
                if (actualStates[face] != "output") {
                    actualStates[face] = getFace(blocksOffsets[index], actualStates[face])
                }

            })
            let blockPerm = mc.BlockPermutation.resolve(tubeId, actualStates)
            ItemTransport(pipe, blocksOffsets, actualStates)
            pipe.setPermutation(blockPerm)

        }
    }
})
mc.world.beforeEvents.itemUseOn.subscribe(data => {
    if (data.itemStack && data.itemStack.typeId == "dnd:wrench" && data.block.typeId == tubeId) {
        mc.system.runTimeout(() => {
            let pipe = data.block
            let ui = new mcui.ModalFormData()
            let faces = [
                "dnd:north",
                "dnd:south",
                "dnd:east",
                "dnd:west",
                "dnd:top",
                "dnd:bottom"
            ]
            if (data.source.isSneaking) {
                ui.title("Close Faces")
                let statesOutput = []
                faces.forEach((face, index) => {
                    if (pipe.permutation.getState(face) == "open" || pipe.permutation.getState(face) == "closed") {
                        statesOutput.push(face)
                        let text = face.replace("dnd:", "")
                        ui.toggle(text.charAt(0).toUpperCase() + text.slice(1), pipe.permutation.getState(face) == "closed" ? true : false)
                    }
                })
                ui.show(data.source).then(r => {
                    let states = statesOutput
                    let actualStates = pipe.permutation.getAllStates()
                    if (r.canceled == false) {
                        states.forEach((state, index) => {
                            actualStates[state] = r.formValues[index] == true ? "closed" : "open"
                        })
                        let block = mc.BlockPermutation.resolve(pipe.typeId, actualStates)
                        data.block.setPermutation(block)
                    }
                })
            } else {
                ui.title("Ouput Faces")
                let statesOutput = []
                faces.forEach((face, index) => {
                    if (pipe.permutation.getState(face) == "open" || pipe.permutation.getState(face) == "output") {
                        statesOutput.push(face)
                        let text = face.replace("dnd:", "")
                        ui.toggle(text.charAt(0).toUpperCase() + text.slice(1), pipe.permutation.getState(face) == "output" ? true : false)
                    }
                })
                ui.show(data.source).then(r => {
                    let states = statesOutput
                    let actualStates = pipe.permutation.getAllStates()
                    if (r.canceled == false) {
                        states.forEach((state, index) => {
                            actualStates[state] = r.formValues[index] == true ? "output" : "open"
                        })
                        let block = mc.BlockPermutation.resolve(pipe.typeId, actualStates)
                        data.block.setPermutation(block)
                    }
                })
            }
        })
    }

})